////////////////////////////////////////////////////////////////////////////////
// File    : Tree.cpp
// Content : a tree implementation
// History : 09-Jun-2005 - [rb] (re)creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include "Tree.h"

#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGGElement.hpp>
#include <w3c/svg/SVGUseElement.hpp>
#include <w3c/svg/SVGRectElement.hpp>

#include <w3c/svg/SVGTransforms.hpp>
#include <w3c/svg/SVGTransformable.hpp>

#include <hsm/svgl/Svgl.h>


/* Node implementation ********************************************************/

Node::Node(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w) {
	// retriving background & height
	hsm::svgl::Childs(elem).get(background);
	height = background->getHeight().getValue();
	
	// retriving parent
	hsm::svgl::getAttribute(elem, "hsm-arg:parent", parent);
	translateAllBrothers(height);
}

float Node::getHeight() {
	return height;
}

void Node::translateAllBrothers(float dy) {
	translateBrothers(dy);
	if((parent != 0) && (parent->isOpen)) {
		parent->translateAllBrothers(dy);
	}
}

void Node::translateBrothers(float dy) {
	dom::Node *sibling = elem->getNextSibling();
	svg::SVGTransformable *transSibling;
	while((transSibling = hsm::svgl::getSibling< svg::SVGTransformable * >(sibling)) != 0) {
		svg::SVGTranslateTransform *trans = transSibling->getTranslate(true);
		trans->ty += dy;
	}
	if(parent != 0) parent->resizeBackground(dy);
}

const hsm::SVGLBehaviour::Factory::Register< Node > Node::registration("node");


/* Tree class implementation **************************************************/

Tree::Tree(svg::SVGElement *e, hsm::SVGLWindow *w) : Node(e, w) {
	// retrieving the various svg parts
	hsm::svgl::Childs(elem).get(handle)
	                       .get(content);
	hsm::svgl::getAttribute(elem, "hsm-arg:open", isOpen);
	
	// initializing content
	dom::Node *child = content->getFirstChild();
	svg::SVGGElement *gChild;
	while((gChild = hsm::svgl::getSibling< svg::SVGGElement * >(child)) != 0) {
		hsm::svgl::setAttribute(gChild, "hsm-arg:parent", this);
	}
	window->instanciateBehaviour(content);

	// initializing handle
	hsm::svgl::setAttribute(handle, "hsm:behaviour", "treeOpener");
	hsm::svgl::setAttribute(handle, "hsm-arg:parent", this);
	window->instanciateBehaviour(handle);
}


float Tree::getHeight() {
	float res = height;
	if(isOpen) {
		dom::Node *child = content->getFirstChild();
		svg::SVGGElement *gChild;
		while((gChild = hsm::svgl::getSibling< svg::SVGGElement * >(child)) != 0) {
			Node *node = dynamic_cast< Node * >(hsm::SVGLBehaviour::Factory::get(gChild));
			if(node != 0) {
				res += node->getHeight();
			}
		}
	}
	return res;
}

void Tree::resizeBackground(float dHeight) {
	background->setHeight(background->getHeight().getValue() + dHeight);
}


void Tree::open() {
	isOpen = true;
	
	// refreshing display
	window->lock();

	handle->setHref("#opened");
	elem->appendChild(content);

	float totalHeight = getHeight();
	resizeBackground(totalHeight - height);
	translateAllBrothers(totalHeight - height);

	window->unlock();
}

void Tree::close() {
	// updating display
	window->lock();
	
	handle->setHref("#closed");
	elem->removeChild(content);
	background->setHeight(height);

	translateAllBrothers(height - getHeight());

	window->unlock();

	isOpen = false;
}


const hsm::SVGLBehaviour::Factory::Register< Tree > Tree::registration("tree");
