////////////////////////////////////////////////////////////////////////////////
// File    : Event.h
// Content : an event class
// History : 22-Oct-2001 - [rb] creation
//           26-Sep-2002 - [rb] changed Message implementation from enum
//                              to unsigned long to allow custom events
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Event.h
/// Library events.


#if !defined(HSM_EVENT_H)
#define HSM_EVENT_H


/* includes *******************************************************************/

#include <ostream>


namespace hsm {

/* forward declarations *******************************************************/

	class Component;


/* types **********************************************************************/

	#define NEW        Message(0) ///< message sent when a composite component has a new children
	#define DELETED    Message(1) ///< message sent when a component is destroyed
	#define CHANGE     Message(2) ///< message sent to notify a change
	#define INTERNAL   Message(3) ///< message sent for internal communication purposes

	/// stores various usefull datas
	union Data {
	public :
		void  *pVoid;  ///< a pointer
		char   aChar;  ///< a char
		bool   aBool;  ///< a bool
		int    aInt;   ///< a int
		long   aLong;  ///< a long
		float  aFloat; ///< a float
		
		Component *pComponent; ///< a hsm::Component

	public :
		/// \{
		/// constructor
		Data(void *p) : pVoid(p)  {}
		Data(char c)  : aChar(c)  {}
		Data(bool b)  : aBool(b)  {}
		Data(int i)   : aInt(i)   {}
		Data(long l)  : aLong(l)  {}
		Data(float f) : aFloat(f) {}

		Data(Component *c) : pComponent(c) {}
		/// \}
		
		/// \{
		/// casts the data
		/// \return the needed data
		operator void *() const { return pVoid; }
		operator char() const   { return aChar; }
		operator bool() const   { return aBool; }
		operator int() const    { return aInt; }
		operator long() const   { return aLong; }
		operator float() const  { return aFloat; }
		
		operator Component *() const { return pComponent; }
		/// \}
	};


/* Event struct declaration ***************************************************/

	/// libhsm event
	struct Event {
		typedef unsigned long Message; ///< message type

		Component *sender;  ///< sender of the event
		Message    message; ///< message sent
		Data		  data;    ///< data
		void *     args;    ///< arguments

		/// copy constructor
		/// \param e the event to clone
		Event(const Event &e) : sender(e.sender), message(e.message), data(e.data), args(args) {}
		
		/// constructor
		/// \param s sender
		/// \param m message
		/// \param d data
		/// \param a arguments
		Event(Component *s = 0, Message m = CHANGE, Data d = 0, void *a = 0) : sender(s), message(m), data(d), args(a) {}
		
		/// destructor
		~Event() {}

		/// serializes an event into a stream
		/// \param os output stream
		/// \param event the event
		/// \return the stream
		friend std::ostream &operator<<(std::ostream &os, const Event &event);
	};


} // namespace hsm


#endif // !defined(HSM_EVENT_H)