 ////////////////////////////////////////////////////////////////////////////////
// File    : SVGLBehaviour.cpp
// Content : classes for supporting interaction with svgl objects
// History : 25-Aug-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/Svgl.h>

#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGTransforms.hpp>
#include <w3c/svg/SVGTransformable.hpp>

#include <hsm/util/Debug.h>


namespace hsm {

/* SVGLBehaviour class implementation ******************************************/

	SVGLBehaviour::SVGLBehaviour(svg::SVGElement *e, SVGLWindow *w) : elem(e), window(w) {
		(*SVGLBehaviour::Factory::behaviours)[elem] = this;	
	}
	
	SVGLBehaviour::~SVGLBehaviour() {
		SVGLBehaviour::Factory::behaviours->erase(elem);	
	}
	
	
/* SVGLEnterLeave class implementation ****************************************/

	SVGLEnterLeave::SVGLEnterLeave(svg::SVGElement *e, SVGLWindow *w) : SVGLBehaviour(e, w) {
		stylable = dynamic_cast< svg::SVGStylable * >(elem);
	}

	void SVGLEnterLeave::enter() {
		if(stylable != 0) {
			window->lock();
			strokeWidth = stylable->getStrokeWidth().getValue();
			stylable->setStrokeWidth(2 * strokeWidth);
			window->unlock();
		}
	}
	
	void SVGLEnterLeave::leave() {
		if(stylable != 0) {
			window->lock();
			stylable->setStrokeWidth(strokeWidth);	
			window->unlock();
		}
	}


/* SVGLTranslate class implementation *****************************************/

	SVGLTranslate::SVGLTranslate(svg::SVGElement *e, SVGLWindow *w) : SVGLBehaviour(e, w) {
		svg::SVGTransformable *transformable = dynamic_cast< svg::SVGTransformable * >(elem);
		tra = transformable->getTranslate(true);
	}

	void SVGLTranslate::translate(const Point &dp) {
		window->lock();
//		svgl::getGlobalTransform(elem, sx, sy, tx, ty, angle);
//		float ca = cos(M_PI * angle / 180.);
//		float sa = sin(M_PI * angle / 180.);
//		float dtx = p/Point::X / sx;
//		float dty = p/Point::Y / sy;
//		
//		tra->tx += ca * dtx + sa * dty;
//		tra->ty += ca * dty - sa * dtx;

		svgl::getGlobalTransform(elem, sx, sy, tx, ty);
		tra->tx += dp/Point::X / sx;
		tra->ty += dp/Point::Y / sy;

		window->unlock();
	}


/* SVGLRotate class implementation ********************************************/

	SVGLRotate::SVGLRotate(svg::SVGElement *e, SVGLWindow *w) : SVGLBehaviour(e, w) {
		svg::SVGTransformable *transformable = dynamic_cast< svg::SVGTransformable * >(elem);
		rot = transformable->getRotate(true);
	}

	void SVGLRotate::rotate(float da /* = 0. */) {
		window->lock();
		rot->angle += da;
		window->unlock();
	}


/* SVGLZoom class implementation **********************************************/

	SVGLZoom::SVGLZoom(svg::SVGElement *e, SVGLWindow *w) : SVGLBehaviour(e, w) {
		svg::SVGTransformable *transformable = dynamic_cast< svg::SVGTransformable * >(elem);
		sca = transformable->getScale(true);
		tra = transformable->getTranslate(true);
	}
	
	void SVGLZoom::scale(float ds /* = 1. */) {
		window->lock();
		// [ToDo] - find the correct translation to perform the actual zoom centered
		//          on the picking point even after rotations.
		svgl::getGlobalTransform(elem, sx, sy, tx, ty);
		sca->sx *= ds; sca->sy *= ds;
		tra->tx *= ds; tra->ty *= ds;
		ds -= 1;
		tra->tx += (tx - pick/Point::X) * ds / sx;
		tra->ty += (ty - pick/Point::Y) * ds / sy;
				
		window->unlock();
	}


} // namespace hsm
