////////////////////////////////////////////////////////////////////////////////
// File    : SVGLWindow.cpp
// Content : Window class with svgl support
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <SVGLWindow.h>

#include <hsm/component/Point.h>
#include <hsm/svgl/Svgl.h>
#include <hsm/svgl/SVGLBehaviour.h>

#include <svgl/Parser.hpp>
#include <svgl/InitHelper.hpp>
#include <svgl/DisplayManager.hpp>
#include <svgl/PickManager.hpp>


namespace hsm {

/* SVGLWindow class implementation ********************************************/

	SVGLWindow::SVGLWindow(const Id &id /* = Id() */) : GLWindow(id) {
		svglDisplayManager = ::svgl::InitHelper::get()->displayManager;
		svglPickManager    = ::svgl::InitHelper::get()->pickManager;

		setDocument(0);

		svgContent = new svg::SVGGElement(svgDocument);
		svgRoot->appendChild(svgContent);
		svgFeedback = new svg::SVGGElement(svgDocument);
		svgRoot->appendChild(svgFeedback);
		svgCursors = new svg::SVGGElement(svgDocument);
		svgRoot->appendChild(svgCursors);
	}

	SVGLWindow::~SVGLWindow() {
		// [ToDo] destroy svgl things
	}

	void SVGLWindow::init() {}
	
	void SVGLWindow::display() {
		glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		svglDisplayManager->display(svgRoot);
	}
	
	void SVGLWindow::reshape(unsigned int width, unsigned int height) {
		svglDisplayManager->reshape(width, height);
	}


	void SVGLWindow::setDocument(const std::string &docName) {
		::svgl::Parser parser;
		svg::SVGDocument *document = parser.parse(docName.c_str());
		setDocument(document);
	}
	
	void SVGLWindow::setDocument(svg::SVGDocument *doc) {
		if(doc == 0) {
			doc = new svg::SVGDocument;
			svg::SVGSVGElement *root = new svg::SVGSVGElement(doc);
			root->setWidth(int(*size/Point::X));
			root->setHeight(int(*size/Point::Y));
			doc->appendChild(root);
		}
		
		lock();
		svgDocument = doc;
		svgRoot = const_cast< svg::SVGSVGElement * >(doc->getRootElement());
		
		svgContent = new svg::SVGGElement(svgDocument);
		dom::Node *child = svgRoot->getFirstChild();
		while(child != 0) {
			dom::Node *nextChild = child->getNextSibling();
			svgRoot->removeChild(child);
			svgContent->appendChild(child);
			child = nextChild;
		}
		
		// relies on the svgl garbage collection
		svgRoot->appendChild(svgContent);
		svgFeedback = new svg::SVGGElement(svgDocument);
		svgRoot->appendChild(svgFeedback);
		svgCursors = new svg::SVGGElement(svgDocument);
		svgRoot->appendChild(svgCursors);
		
		hsm::svgl::registerIds(svgDocument, svgRoot);
		svgDocument->updateStyle();
		unlock();
		
		setSize((int)(svgRoot->getWidth().getValue()), (int)(svgRoot->getHeight().getValue()));
		initBehaviours();
	}
	
	void SVGLWindow::setContentBehaviour(const std::string &behaviour) {
		hsm::svgl::setAttribute(svgContent, "hsm:behaviour", behaviour);
		initBehaviours();
	}


	void SVGLWindow::initBehaviours() {
		SVGLBehaviour::Factory::ensureLoading();
		lock();
		instanciateBehaviour(svgRoot);
		unlock();
	}
	
	void SVGLWindow::instanciateBehaviour(svg::SVGElement *elem) {
		unicode::String *type = elem->getAttribute(unicode::String::createString("hsm:behaviour"));
		if((type != 0) && (SVGLBehaviour::Factory::get(elem) == 0)) {
			SVGLBehaviour::Factory::create(type->transcode(), elem, this);
		}
		dom::Node *child = elem->getFirstChild();
		svg::SVGElement *svgChild;
		while((svgChild = hsm::svgl::getSibling< svg::SVGElement * >(child)) != 0) {
			instanciateBehaviour(svgChild);
		}
	}


} // namespace hsm
