////////////////////////////////////////////////////////////////////////////////
// File    : Timer.h
// Content : a timer class
// History : 21-Dec-2001 - [rb] creation
//           18-Jan-2002 - [rb] adapted to pass data to the Functor object
//           10-Feb-2003 - [rb] changed constructor interface
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Timer.h
/// A timer class.


#if !defined(HSM_TIMER_H)
#define HSM_TIMER_H


/* includes *******************************************************************/

#if defined(MACOSX)
#  include <Carbon/Carbon.h>
#elif defined(WIN32)
#  define _WIN32_WINNT 0x0400
#  include <windows.h>
#endif

#include <map>


namespace hsm {

/* forward declarations *******************************************************/

	class Functor;


/* Timer class definition *****************************************************/

	/// implements a timer
	class Timer {

		/// os dependent declarations
		struct os {
		#if defined(MACOSX)
			typedef void              TimerCb(EventLoopTimerRef, void *); ///< timer callback
			typedef EventLoopTimerRef Id;                                 ///< timer descriptor
		#elif defined(WIN32)
			typedef VOID CALLBACK     TimerCb(HWND, UINT, UINT, DWORD);   ///< timer callback
			typedef UINT              Id;                                 ///< timer descriptor
		#endif

			/// installs a system timer
			/// \param d the delay in ms
			/// \return the system Id of the timer created
			static Id	installTimer(unsigned int d);
			
			/// removes a system timer
			/// \param id the system Id of the timer to remove
			static void removeTimer(Id id);
			
		protected :
			static TimerCb   timerCb;
		};

		friend struct os;

	private :
		typedef std::map< os::Id, Timer * > Map;
		static Map timerMap;

	private :
		os::Id  timerId;

		int delay;

		Functor *timerFn;
		void    *data;

		bool fired;
		bool enabled;

	public :
		/// constructor
		/// \param ms the time before the timer fires in ms
		/// \param tfn a pointer on a functor to call back after the elapsed time
		/// \param d a \c void pointer on the data to pass to the callback
		Timer(int ms = 0, Functor *tfn = 0, void *d = 0);
		
		/// destructor
		/// the destructor disarm the timer if the time has not been elpased
		~Timer();

		void stop();  ///< disarms the timer
		 /// rearms the timer
		 /// \param ms the time before the timer fires in ms (0 keeps the old
		 /// value)
		void rearm(int ms = 0);

		/// test if the timer has already fired
		/// \retval true if the timer has fired
		/// \retval false otherwise
		bool hasFired() const;
		inline operator const bool() const { return hasFired(); } ///< see hasFired()

	private :
		void install();
		void remove();

		void fire();
	};


} // namespace hsm


#endif // !defined(HSM_TIMER_H)
